module Injectable
  # Memoizes the Dependencies generated by Dependency based on DependencyGraph
  class DependenciesProxy
    attr_reader :graph, :namespace

    def initialize(graph:, namespace: nil)
      @graph = graph
      @namespace = namespace
      @instances = {}
    end

    # Get the instance of the dependency +name+
    def get(name)
      @instances[name] ||= graph[name].instance(args: memoized_dependencies_of(name), namespace: namespace)
    end

    private

    def memoized_dependencies_of(name)
      return [] if dependencies_of(name).empty?

      dependencies_of(name).each_with_object({}) { |dep, hash| hash[dep] = get(dep) }
    end

    def dependencies_of(name)
      graph[name].depends_on
    end
  end
end
